package functional.graph.sameEdges;

class Graph {
	/*@ nullable @*/ EdgeList edges;
	/*@ nullable @*/ NodeList nodes;

	/*@ 
	 @ requires (\forall Node n; true; n.inEdges!=null && n.outEdges!=null);
	 @ requires (\forall Node n1, n2; n1 != n2; n1.nodeID != n2.nodeID);
	 @ requires (\forall Edge e1, e2; e1 != e2; e1.edgeID != e2.edgeID);
	 
	 @ requires (\forall NodeList l; true; l.count >= 0 && l.entries != null && l.count <= l.entries.length
	 			&& (\forall int i; i>=0 && i<l.count; l.entries[i] != null));
	
	 @ requires (\forall EdgeList l; true; l.count >= 0 && l.entries != null && l.count <= l.entries.length
	 			&& (\forall int i; i>=0 && i<l.count; l.entries[i] != null));
	 
	 @ requires edges != null && nodes != null && nl != null;
	 
     @ ensures \old(nl.count) == 0 ==> (\forall int i; 0<=i && i<edges.count; (\exists int j; 0<=j && j<\old(edges.count); edges.entries[i]==\old(edges.entries[j])));
	 @ diverges true;
	 @ assignable \everything;
	 @*/
	void wrapper(/*@ nullable @*/ NodeList nl) {
		remove(nl);
	}

	void remove(/*@ nullable @*/ NodeList nl) {
		// remove nodes
		{
			Node[] newEntries = new Node[nodes.count];

			int lastI = 0;
			int index = 0;
			int nodesCount = this.nodes.count;
			/*@		 
			 @ maintaining true;
			 @ assignable newEntries[*];
			 @*/
			while (index < nodesCount) {
				Node n = this.nodes.entries[index];

				boolean containsNode = false;
				int i = 0;
				int nlCount = nl.count;
				/*@		 
				 @ maintaining true;
				 @ assignable \strictly_nothing;
				 @*/
				while (i < nlCount) {
					if (nl.entries[i].nodeID == n.nodeID) {
						containsNode = true;
						break;
					}
					i = i + 1;
				}

				if (!containsNode) {
					newEntries[lastI] = n;
					lastI = lastI + 1;
				}
				index = index + 1;
			}
			nodes.entries = newEntries;
			nodes.count = lastI;
		}

		// remove outEdges and inEdges
		EdgeList el = edges;
		int j = 0;
		int nodesCount = this.nodes.count;
		/*@ maintaining 0<=j;
		  @ maintaining (\forall int k; 0<=k && k<edges.count; (\exists int l; 0<=l && l<\old(edges.count); edges.entries[k]==\old(edges.entries[l])));
		  @ assignable edges.entries, edges.count;
		  @*/
		while (j < nodesCount) {
			Node n = this.nodes.entries[j];

			// remove outEdges and outEdges
			{
				Edge[] newEntries = new Edge[el.count];
				int lastI = 0;
				int oj = 0;
				int elCount = el.count;
				/*@		 
				 @ maintaining (\forall int a; 0<=a && a<lastI; (\exists int b; 0<=b && b<oj; newEntries[a] == el.entries[b]));
                 @ maintaining 0<=oj && oj<=el.count;
                 @ maintaining 0<=lastI && lastI<=oj;
				 @ assignable newEntries[*];
				 @*/
				while (oj < elCount) {
					Edge e = el.entries[oj];

					boolean containsE = false;
					int iE = 0;
					int outEdgeCount = n.outEdges.count;
					/*@ loop_invariant 0<=iE && iE<= outEdgeCount;
					  @ loop_invariant (\forall int k; 0<=k && k<iE; n.outEdges.entries[k].edgeID!=e.edgeID);					
					  @ assignable \strictly_nothing;
					  @*/
					while (iE < outEdgeCount) {
						if (n.outEdges.entries[iE].edgeID == e.edgeID) {
							containsE = true;
							break;
						}
						iE = iE + 1;
					}
					iE = 0;
					int inEdgeCount = n.inEdges.count;
					if (!containsE){
						/*@ loop_invariant 0<=iE && iE<= inEdgeCount;
						  @ loop_invariant (\forall int k; 0<=k && k<iE; n.inEdges.entries[k].edgeID!=e.edgeID);					
						  @ assignable \strictly_nothing;
						  @*/
						while (iE < inEdgeCount) {
							if (n.inEdges.entries[iE].edgeID == e.edgeID) {
								containsE = true;
								break;
							}
							iE = iE + 1;
						}
					}

					if (!containsE) {
						newEntries[lastI] = e;
						lastI = lastI + 1;
					}
					oj = oj + 1;
				}
				el.entries = newEntries;
				el.count = lastI;
			}

			j = j + 1;
		}
	}
}