package abstraction.dictionary.ArrayIndexOutOfBoundsException;

import abstraction.RuntimeException;

final class Dictionary {
	/*@ nullable @*/Key[] keys;
	/*@ nullable @*/Value[] values;

	/*@ requires !RuntimeException.NullPointerException;
	  @ requires !RuntimeException.ArrayIndexOutOfBoundsException;
	  @ requires !RuntimeException.NegativeArraySizeException;
	  @ requires keys != null && values != null;
	  @ requires (\forall int i,j; 0<=i && i<j && j<keys.length; keys[i] != keys[j]);
	  @ requires keys.length == values.length;
	  @ requires 0<=keys.length && keys.length<Integer.MAX_VALUE;
	  @ ensures !RuntimeException.ArrayIndexOutOfBoundsException;
	  @ diverges true;
	  @ assignable \everything;
	  @*/
	void wrapper(/*@ nullable @*/ Key k, /*@ nullable @*/ Value v) {
		put(k, v);
	}

	void put(/*@ nullable @*/ Key k, /*@ nullable @*/ Value v) {
		int pos = -1;
		int i = 0;
		int keyLen = this.keys.length;
                //@ loop_invariant 0 <= i && i <= keyLen && keyLen == keys.length && keys.length == values.length;
                //@ loop_invariant pos <= i && pos < keyLen;
                //@ assignable i, pos;
		while (i < keyLen) {
			if (i < 0 || i >= keyLen) {
				RuntimeException.ArrayIndexOutOfBoundsException = true;
				return;
			}
			if (this.keys[i] == k) {
				pos = i;
				break;
			}
			i = i + 1;
		}

		if (pos >= 0) {
			int vLen = this.values.length;
			if (pos < 0 || pos >= vLen) {
				RuntimeException.ArrayIndexOutOfBoundsException = true;
				return;
			}
			this.values[pos] = v;
		} else {
			Key[] oldKs = this.keys;
			int oldKLen = oldKs.length;
			int iK = 0;
			this.keys = new Key[oldKLen + 1];
			if (oldKLen < 0 || oldKLen >= this.keys.length) {
				RuntimeException.ArrayIndexOutOfBoundsException = true;
				return;
			}
			this.keys[oldKLen] = k;
                        //@ loop_invariant 0 <= iK && iK <= oldKs.length && oldKs.length + 1 == keys.length;
                        //@ assignable iK, keys[*];
			while (iK < oldKLen) {
				if (iK < 0 || iK >= oldKs.length) {
					RuntimeException.ArrayIndexOutOfBoundsException = true;
					return;
				}
				if (iK < 0 || iK >= this.keys.length) {
					RuntimeException.ArrayIndexOutOfBoundsException = true;
					return;
				}
				this.keys[iK] = oldKs[iK];
				iK = iK + 1;
			}

			Value[] oldVs = this.values;
			int oldVLen = oldVs.length;
			int iV = 0;
			this.values = new Value[oldVLen + 1];
			if (oldVLen < 0 || oldVLen >= this.values.length) {
				RuntimeException.ArrayIndexOutOfBoundsException = true;
				return;
			}
			this.values[oldVLen] = v;
                        //@ loop_invariant 0 <= iV && iV <= oldVs.length && oldVs.length +1 == values.length;
                        //@ assignable iV, values[*];
			while (iV < oldVLen) {
				if (iV < 0 || iV >= oldVs.length) {
					RuntimeException.ArrayIndexOutOfBoundsException = true;
					return;
				}
				if (iV < 0 || iV >= this.values.length) {
					RuntimeException.ArrayIndexOutOfBoundsException = true;
					return;
				}
				this.values[iV] = oldVs[iV];
				iV = iV + 1;
			}
		}
	}
}
