package abstraction.dictionary.NegativeArraySizeException;

import abstraction.RuntimeException;

final class Dictionary {
	/*@ nullable @*/Key[] keys;
	/*@ nullable @*/Value[] values;

	/*@ requires !RuntimeException.NullPointerException;
	  @ requires !RuntimeException.ArrayIndexOutOfBoundsException;
	  @ requires !RuntimeException.NegativeArraySizeException;
	  @ requires keys != null && values != null;
	  @ requires (\forall int i,j; 0<=i && i<j && j<keys.length; keys[i] != keys[j]);
	  @ requires keys.length == values.length;
	  @ requires 0<=keys.length && keys.length<Integer.MAX_VALUE;
	  @ ensures !RuntimeException.NegativeArraySizeException;
	  @ diverges true;
	  @ assignable \everything;
	  @*/
	void wrapper(/*@ nullable @*/ Key k, /*@ nullable @*/ Value v) {
		put(k, v);
	}

	void put(/*@ nullable @*/ Key k, /*@ nullable @*/ Value v) {
		int pos = -1;
		int i = 0;
		int keyLen = this.keys.length;
		/*@ 
          @ maintaining true;
		  @ assignable \strictly_nothing;
		  @*/
		while (i < keyLen) {
			if (this.keys[i] == k) {
				pos = i;
				break;
			}
			i = i + 1;
		}

		if (pos >= 0) {
			this.values[pos] = v;
		} else {
			Key[] oldKs = this.keys;
			int oldKLen = oldKs.length;
			int iK = 0;
			if (oldKLen + 1 < 0) {
				RuntimeException.NegativeArraySizeException = true;
				return;
			}
			this.keys = new Key[oldKLen + 1];
			this.keys[oldKLen] = k;
			/*@ 
			  @ maintaining true;
			  @ assignable this.keys[*];
			 */
			while (iK < oldKLen) {
				this.keys[iK] = oldKs[iK];
				iK = iK + 1;
			}

			Value[] oldVs = this.values;
			int oldVLen = oldVs.length;
			int iV = 0;
			if (oldVLen + 1 < 0) {
				RuntimeException.NegativeArraySizeException = true;
				return;
			}
			this.values = new Value[oldVLen + 1];
			this.values[oldVLen] = v;
			/*@ 
			  @ maintaining true;
			  @ assignable this.values[*];
			 */
			while (iV < oldVLen) {
				this.values[iV] = oldVs[iV];
				iV = iV + 1;
			}
		}
	}
}
