package abstraction.lrs.ArrayIndexOutOfBoundsException;

import abstraction.RuntimeException;

final class LRS {
	int s = 0;
	int t = 0;
	int l = 0;
	/*@ nullable */SuffixArray sa;

	/*@ 
	  @ requires !RuntimeException.NullPointerException;
	  @ requires !RuntimeException.ArrayIndexOutOfBoundsException;
	  @ requires sa!=null && sa.a != null && sa.suffixes != null;
	  @ requires sa.a.length >= 2;
	  @// from class invariants
	  @ requires (\forall int i; 0 <= i && i < sa.a.length; (\exists int j; 0 <= j && j < sa.a.length; sa.suffixes[j]==i));
	  @ requires (\forall int i; 0 <= i && i < sa.a.length; 0 <= sa.suffixes[i] && sa.suffixes[i] < sa.a.length);
	  @ requires (\forall int i; 0 < i && i < sa.a.length; sa.suffixes[i-1] != sa.suffixes[i]);
	  @ requires (\forall int i; 0 < i && i < sa.a.length; 
	  @   (sa.a != null && 0 <= sa.suffixes[i] && sa.suffixes[i] < sa.a.length && 0 <= sa.suffixes[i-1] && sa.suffixes[i-1] < sa.a.length) 
	  @	  ==> (\exists int j; 0 <= j && j < sa.a.length-sa.suffixes[i];
	  @           ((j < sa.a.length-sa.suffixes[i-1] && sa.a[sa.suffixes[i]+j] > sa.a[sa.suffixes[i-1]+j] ) || j == sa.a.length-sa.suffixes[i-1])
	  @               && (\forall int k; 0 <= k && k < j; sa.a[sa.suffixes[i]+k] == sa.a[sa.suffixes[i-1]+k])));
	  @ requires sa.a.length == sa.suffixes.length;
	  @ ensures !RuntimeException.ArrayIndexOutOfBoundsException;
	  @ diverges true;
	  @ assignable \everything;
	  @*/
	void wrapper(/*@ nullable @*/ int[] array) {
		doLRS(array);
	}

	void doLRS(int[] array) {
		int s = 0;
		int t = 0;
		int l = 0;
		int len = sa.a.length;

		int[] suffixA = sa.suffixes;

		int x = 1;
		/*@ maintaining 1<=x && x<=len;
		  @ assignable \strictly_nothing;
		 */
		while (x < len) {
			if (x < 0 || x >= suffixA.length) {
				RuntimeException.ArrayIndexOutOfBoundsException = true;
				return;
			}
			int begin = suffixA[x];

			if (x - 1 < 0 || x - 1 >= suffixA.length) {
				RuntimeException.ArrayIndexOutOfBoundsException = true;
				return;
			}
			int end = suffixA[x - 1];

			int length = 0;
			int[] aA = sa.a;

			int tLength = aA.length;
			/*@
			  @ maintaining true;
			  @ assignable \strictly_nothing;
			 */
			while (begin + length < tLength) {
				if (end + length < tLength){
					if (begin + length < 0 && begin + length >= tLength) {
						RuntimeException.ArrayIndexOutOfBoundsException = true;
						return;
					}
					if (end + length < 0 && end + length >= tLength) {
						RuntimeException.ArrayIndexOutOfBoundsException = true;
						return;
					}
					if (aA[begin + length] == aA[end + length]){
						length = length + 1;
					}else{
						break;
					}
				}else{
					break;
				}
			}

			if (length > l) {
				s = begin;
				t = end;
				l = length;
			}
			x = x + 1;
		}
		this.s = s;
		this.t = t;
		this.l = l;
	}
}
